/*
 * aduc - ad.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 * Copyright (C) 2009..2011  Krzysztof Mazur <krzysiek@podlesie.net>  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/config.h>
#include <tmos/dac.h>
#include <asm/aduc841.h>

/*
 * DACCON
 *	7	MODE	1=8 bit 0= 12 bit  
 *	6	RNG1	DAC1 Range 1= 0-Vdd =0 0-Vref
 *	5	RNG0	DAC0 Range 1= 0-Vdd =0 0-Vref                      
 *	4	CLR1	Clr DAC1 0= force 0V 
 *	3	CLR0	Clr DAC0 0= force 0V
 *	2	SYNC	Update synchronization bit
 *	1	PD1	DAC1 Power Down bit 1= Power ON;0= Power OFF
 *	0	PR0	DAC0 Power Down bit 1= Power ON;0= Power OFF
 */
#define DAC_CNF	0x1f

void dac_init(void)
{
	/* bypass buffer */
	CFG841 |= CFG841_DBUF;
	DACCON = DAC_CNF;
}

void dac_write(unsigned int ch, int16_t x)
{
#if CONFIG_DAC_ADUC841_UNIPOLAR
	if (x < 0)
		x = 0;
	x >>= 3;
#else
	/* convert two's complement value to biased value */
	x += 0x8000;
	x >>= 4;
#endif

	switch (ch) {
	case 0:
		DAC0H = (x >> 8) & 0x0f;
		DAC0L = x & 0xff;
		break;
	case 1:
		DAC1H = (x >> 8) & 0x0f;
		DAC1L = x & 0xff;
		break;
	}
}

device_initcall(dac_init)
