/*
 * tmos - trt.c
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tmos/kernel.h>
#include <tmos/trt.h>
#include <tmos/mb-slave.h>
#include <tmos/jiffies.h>

u16 __trt __trt_regs[TRT_REGS];

#ifdef CONFIG_TRT_SYNC
static jiffies_t trt_next_jiffy;
#endif

#ifdef CONFIG_TRT_TIMEOUT
static jiffies_t wdt;
static __bit timeout = 1;

void trt_check_timeout(void) __reentrant
{
	if (!timeout)
		if (time_after(get_jiffies(), wdt + ((long)
						CONFIG_TRT_TIMEOUT_VALUE * HZ
						+ 999) / 1000))
			timeout = 1;

	if (timeout) {
#ifdef CONFIG_TRT_BDO5
		trt_bdo5_timeout();
#endif
	}
}

void trt_refresh(void) __reentrant
{
	wdt = get_jiffies();
	timeout = 0;
}
#else
#define trt_check_timeout()	do { } while (0)
#endif

static void trt_io_read(void) __reentrant
{
#ifdef CONFIG_TRT_JIFFIES
	trt_jiffies_read();
#endif
#ifdef CONFIG_TRT_ADC
	trt_adc_read();
#endif
#ifdef CONFIG_TRT_BDI32
	trt_bdi32_read();
#endif
#ifdef CONFIG_TRT_BIMP
	trt_bimp_read();
#endif
}

static void trt_io_write(void) __reentrant
{	
#ifdef CONFIG_TRT_DAC
	trt_dac_write();
#endif
#ifdef CONFIG_TRT_BDO5
	trt_bdo5_write();
#endif
}

static void trt_io_xchg(void) __reentrant
{	
#ifdef CONFIG_TRT_MB_SLAVE
	mb_slave_recv();
#endif
}

#ifdef CONFIG_TRT_SYNC
char trt_probe(void)
{
	jiffies_t t = get_jiffies();
	char ret;

	ret = time_after_eq(t, trt_next_jiffy);
	if (ret) {
		trt_next_jiffy += CONFIG_TRT_SYNC_PERIOD;

		/* skip some samples */
		if (time_after_eq(t, trt_next_jiffy)) {
			trt_next_jiffy = t + CONFIG_TRT_SYNC_PERIOD;
//			trt_write_reg(trt_read_reg(16) + 1, 16);
		}
	}
	return ret;
}
#endif

void trt_poll(void) __reentrant
{
#ifdef CONFIG_TRT_SYNC
	if (!trt_probe())
		return;
#endif

//	trt_write_reg(trt_read_reg(17) + 1, 17);
	trt_check_timeout();
	trt_io_read();
	trt_io_write();
	trt_io_xchg();
}

void trt_init(void) __reentrant
{
	unsigned int i;

	for (i = 0; i < TRT_REGS; i++)
		trt_write_reg(0, i);

#ifdef CONFIG_TRT_SYNC
	trt_next_jiffy = get_jiffies();
#endif
#ifdef CONFIG_TRT_MB_SLAVE_MAP
	mb_mapping_init();
#endif
}

subsys_initcall(trt_init)
