/*
 * tmos - delay.h
 *
 * Copyright (C) 2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_DELAY_H
#define __ASM_DELAY_H

#include <tmos/kernel.h>
#include <tmos/config.h>
#include <asm/system.h>

#if CONFIG_DELAY_LPJ
#define loops_per_jiffy		CONFIG_DELAY_LPJ
#else
extern unsigned int loops_per_jiffy;
#define DELAY_AUTODETECT 	1
#endif

#if !CONFIG_DELAY_LPJ || CONFIG_PRINT_BOGOMIPS
void delay_init(void);
#else
#define delay_init() do { } while (0)
#endif

void __delay(unsigned int loops) __naked;

void __udelay(unsigned int n);
void __mdelay(unsigned int n);

#ifdef DELAY_AUTODETECT
#define udelay(n) __udelay(n)
#define mdelay(n) __mdelay(n)

#if CONFIG_JIFFIES_NONE
#error "loops_per_jiffy autocalibration needs jiffies support"
#endif

#else
#define udelay(n) \
	do { \
		if (__builtin_constant_p(n)) { \
			__delay((unsigned long) n * loops_per_jiffy \
					/ (1000000 / HZ) + 1); \
		} else { \
			__udelay(n); \
		} \
	} while (0)

#define mdelay(n) __mdelay(n)
#endif

#endif
